@@ -1,4 +1,4 @@
-#   Copyright 2012-2013 OpenStack, LLC.
+#   Copyright 2012-2013 OpenStack Foundation
 #
 #   Licensed under the Apache License, Version 2.0 (the "License"); you may
 #   not use this file except in compliance with the License. You may obtain
@@ -13,7 +13,7 @@
 #   under the License.
 #
 
-"""Modify Cliff's CommandManager"""
+"""Modify cliff.CommandManager"""
 
 import logging
 import pkg_resources
@@ -25,18 +25,34 @@ LOG = logging.getLogger(__name__)
 
 
 class CommandManager(cliff.commandmanager.CommandManager):
-    """Alters Cliff's default CommandManager behaviour to load additiona
-       command groups after initialization.
+    """Add additional functionality to cliff.CommandManager
+
+    Load additional command groups after initialization
+    Add *_command_group() methods
     """
+
+    def __init__(self, namespace, convert_underscores=True):
+        self.group_list = []
+        super(CommandManager, self).__init__(namespace, convert_underscores)
+
     def _load_commands(self, group=None):
         if not group:
             group = self.namespace
+        self.group_list.append(group)
         for ep in pkg_resources.iter_entry_points(group):
-            LOG.debug('found command %r' % ep.name)
-            self.commands[ep.name.replace('_', ' ')] = ep
+            cmd_name = (
+                ep.name.replace('_', ' ')
+                if self.convert_underscores
+                else ep.name
+            )
+            self.commands[cmd_name] = ep
         return
 
     def add_command_group(self, group=None):
         """Adds another group of command entrypoints"""
         if group:
             self._load_commands(group)
+
+    def get_command_groups(self):
+        """Returns a list of the loaded command groups"""
+        return self.group_list
