@@ -72,7 +72,7 @@ class CreateUser(show.ShowOne):
         return parser
 
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)' % parsed_args)
+        self.log.debug('take_action(%s)', parsed_args)
         identity_client = self.app.client_manager.identity
 
         if parsed_args.project:
@@ -99,9 +99,10 @@ class CreateUser(show.ShowOne):
         # NOTE(dtroyer): The users.create() method wants 'tenant_id' but
         #                the returned resource has 'tenantId'.  Sigh.
         #                We're using project_id now inside OSC so there.
-        user._info.update(
-            {'project_id': user._info.pop('tenantId')}
-        )
+        if 'tenantId' in user._info:
+            user._info.update(
+                {'project_id': user._info.pop('tenantId')}
+            )
 
         info = {}
         info.update(user._info)
@@ -123,7 +124,7 @@ class DeleteUser(command.Command):
         return parser
 
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)' % parsed_args)
+        self.log.debug('take_action(%s)', parsed_args)
         identity_client = self.app.client_manager.identity
 
         user = utils.find_resource(
@@ -155,7 +156,8 @@ class ListUser(lister.Lister):
         return parser
 
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)' % parsed_args)
+        self.log.debug('take_action(%s)', parsed_args)
+        identity_client = self.app.client_manager.identity
 
         def _format_project(project):
             if not project:
@@ -165,6 +167,14 @@ class ListUser(lister.Lister):
             else:
                 return project
 
+        project = None
+        if parsed_args.project:
+            project = utils.find_resource(
+                identity_client.tenants,
+                parsed_args.project,
+            )
+            project = project.id
+
         if parsed_args.long:
             columns = (
                 'ID',
@@ -183,14 +193,20 @@ class ListUser(lister.Lister):
             # Cache the project list
             project_cache = {}
             try:
-                for p in self.app.client_manager.identity.tenants.list():
+                for p in identity_client.tenants.list():
                     project_cache[p.id] = p
             except Exception:
                 # Just forget it if there's any trouble
                 pass
         else:
             columns = column_headers = ('ID', 'Name')
-        data = self.app.client_manager.identity.users.list()
+        data = identity_client.users.list(tenant_id=project)
+
+        if parsed_args.project:
+            d = {}
+            for s in data:
+                d[s.id] = s
+            data = d.values()
 
         if parsed_args.long:
             # FIXME(dtroyer): Sometimes user objects have 'tenant_id' instead
@@ -262,7 +278,7 @@ class SetUser(command.Command):
         return parser
 
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)' % parsed_args)
+        self.log.debug('take_action(%s)', parsed_args)
         identity_client = self.app.client_manager.identity
 
         if parsed_args.password_prompt:
@@ -328,7 +344,7 @@ class ShowUser(show.ShowOne):
         return parser
 
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)' % parsed_args)
+        self.log.debug('take_action(%s)', parsed_args)
         identity_client = self.app.client_manager.identity
 
         user = utils.find_resource(
