@@ -14,14 +14,14 @@
 #
 
 import copy
+import mock
 
 from openstackclient.identity.v3 import project
 from openstackclient.tests import fakes
 from openstackclient.tests.identity.v3 import fakes as identity_fakes
-from openstackclient.tests.identity.v3 import test_identity
 
 
-class TestProject(test_identity.TestIdentityv3):
+class TestProject(identity_fakes.TestIdentityv3):
 
     def setUp(self):
         super(TestProject, self).setUp()
@@ -71,13 +71,14 @@ class TestProjectCreate(TestProject):
 
         # Set expected values
         kwargs = {
+            'name': identity_fakes.project_name,
+            'domain': None,
             'description': None,
             'enabled': True,
         }
-        # ProjectManager.create(name, domain, description=, enabled=, **kwargs)
+        # ProjectManager.create(name=, domain=, description=,
+        #                       enabled=, **kwargs)
         self.projects_mock.create.assert_called_with(
-            identity_fakes.project_name,
-            None,
             **kwargs
         )
 
@@ -110,13 +111,14 @@ class TestProjectCreate(TestProject):
 
         # Set expected values
         kwargs = {
+            'name': identity_fakes.project_name,
+            'domain': None,
             'description': 'new desc',
             'enabled': True,
         }
-        # ProjectManager.create(name, domain, description=, enabled=, **kwargs)
+        # ProjectManager.create(name=, domain=, description=,
+        #                       enabled=, **kwargs)
         self.projects_mock.create.assert_called_with(
-            identity_fakes.project_name,
-            None,
             **kwargs
         )
 
@@ -149,13 +151,14 @@ class TestProjectCreate(TestProject):
 
         # Set expected values
         kwargs = {
+            'name': identity_fakes.project_name,
+            'domain': identity_fakes.domain_id,
             'description': None,
             'enabled': True,
         }
-        # ProjectManager.create(name, domain, description=, enabled=, **kwargs)
+        # ProjectManager.create(name=, domain=, description=,
+        #                       enabled=, **kwargs)
         self.projects_mock.create.assert_called_with(
-            identity_fakes.project_name,
-            identity_fakes.domain_id,
             **kwargs
         )
 
@@ -170,6 +173,45 @@ class TestProjectCreate(TestProject):
         )
         self.assertEqual(data, datalist)
 
+    def test_project_create_domain_no_perms(self):
+        arglist = [
+            '--domain', identity_fakes.domain_id,
+            identity_fakes.project_name,
+        ]
+        verifylist = [
+            ('domain', identity_fakes.domain_id),
+            ('enable', False),
+            ('disable', False),
+            ('name', identity_fakes.project_name),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        mocker = mock.Mock()
+        mocker.return_value = None
+
+        with mock.patch("openstackclient.common.utils.find_resource", mocker):
+            columns, data = self.cmd.take_action(parsed_args)
+
+        # Set expected values
+        kwargs = {
+            'name': identity_fakes.project_name,
+            'domain': identity_fakes.domain_id,
+            'description': None,
+            'enabled': True,
+        }
+        self.projects_mock.create.assert_called_with(
+            **kwargs
+        )
+        collist = ('description', 'domain_id', 'enabled', 'id', 'name')
+        self.assertEqual(columns, collist)
+        datalist = (
+            identity_fakes.project_description,
+            identity_fakes.domain_id,
+            True,
+            identity_fakes.project_id,
+            identity_fakes.project_name,
+        )
+        self.assertEqual(data, datalist)
+
     def test_project_create_enable(self):
         arglist = [
             '--enable',
@@ -187,13 +229,14 @@ class TestProjectCreate(TestProject):
 
         # Set expected values
         kwargs = {
+            'name': identity_fakes.project_name,
+            'domain': None,
             'description': None,
             'enabled': True,
         }
-        # ProjectManager.create(name, domain, description=, enabled=, **kwargs)
+        # ProjectManager.create(name=, domain=, description=,
+        #                       enabled=, **kwargs)
         self.projects_mock.create.assert_called_with(
-            identity_fakes.project_name,
-            None,
             **kwargs
         )
 
@@ -225,13 +268,55 @@ class TestProjectCreate(TestProject):
 
         # Set expected values
         kwargs = {
+            'name': identity_fakes.project_name,
+            'domain': None,
             'description': None,
             'enabled': False,
         }
-        # ProjectManager.create(name, domain, description=, enabled=, **kwargs)
+        # ProjectManager.create(name=, domain=,
+        #                       description=, enabled=, **kwargs)
         self.projects_mock.create.assert_called_with(
+            **kwargs
+        )
+
+        collist = ('description', 'domain_id', 'enabled', 'id', 'name')
+        self.assertEqual(columns, collist)
+        datalist = (
+            identity_fakes.project_description,
+            identity_fakes.domain_id,
+            True,
+            identity_fakes.project_id,
             identity_fakes.project_name,
-            None,
+        )
+        self.assertEqual(data, datalist)
+
+    def test_project_create_property(self):
+        arglist = [
+            '--property', 'fee=fi',
+            '--property', 'fo=fum',
+            identity_fakes.project_name,
+        ]
+        verifylist = [
+            ('property', {'fee': 'fi', 'fo': 'fum'}),
+            ('name', identity_fakes.project_name),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        # DisplayCommandBase.take_action() returns two tuples
+        columns, data = self.cmd.take_action(parsed_args)
+
+        # Set expected values
+        kwargs = {
+            'name': identity_fakes.project_name,
+            'domain': None,
+            'description': None,
+            'enabled': True,
+            'fee': 'fi',
+            'fo': 'fum',
+        }
+        # ProjectManager.create(name=, domain=, description=,
+        #                       enabled=, **kwargs)
+        self.projects_mock.create.assert_called_with(
             **kwargs
         )
 
@@ -337,6 +422,59 @@ class TestProjectList(TestProject):
         ), )
         self.assertEqual(tuple(data), datalist)
 
+    def test_project_list_domain(self):
+        arglist = [
+            '--domain', identity_fakes.domain_name,
+        ]
+        verifylist = [
+            ('domain', identity_fakes.domain_name),
+        ]
+
+        self.domains_mock.get.return_value = fakes.FakeResource(
+            None,
+            copy.deepcopy(identity_fakes.DOMAIN),
+            loaded=True,
+        )
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        # DisplayCommandBase.take_action() returns two tuples
+        columns, data = self.cmd.take_action(parsed_args)
+        self.projects_mock.list.assert_called_with(
+            domain=identity_fakes.domain_id)
+
+        collist = ('ID', 'Name')
+        self.assertEqual(columns, collist)
+        datalist = ((
+            identity_fakes.project_id,
+            identity_fakes.project_name,
+        ), )
+        self.assertEqual(tuple(data), datalist)
+
+    def test_project_list_domain_no_perms(self):
+        arglist = [
+            '--domain', identity_fakes.domain_id,
+        ]
+        verifylist = [
+            ('domain', identity_fakes.domain_id),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        mocker = mock.Mock()
+        mocker.return_value = None
+
+        with mock.patch("openstackclient.common.utils.find_resource", mocker):
+            columns, data = self.cmd.take_action(parsed_args)
+
+        self.projects_mock.list.assert_called_with(
+            domain=identity_fakes.domain_id)
+        collist = ('ID', 'Name')
+        self.assertEqual(columns, collist)
+        datalist = ((
+            identity_fakes.project_id,
+            identity_fakes.project_name,
+        ), )
+        self.assertEqual(tuple(data), datalist)
+
 
 class TestProjectSet(TestProject):
 
@@ -489,6 +627,35 @@ class TestProjectSet(TestProject):
             **kwargs
         )
 
+    def test_project_set_property(self):
+        arglist = [
+            '--property', 'fee=fi',
+            '--property', 'fo=fum',
+            identity_fakes.project_name,
+        ]
+        verifylist = [
+            ('property', {'fee': 'fi', 'fo': 'fum'}),
+            ('project', identity_fakes.project_name),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        result = self.cmd.run(parsed_args)
+        self.assertEqual(result, 0)
+
+        # Set expected values
+        kwargs = {
+            'description': identity_fakes.project_description,
+            'domain': identity_fakes.domain_id,
+            'enabled': True,
+            'name': identity_fakes.project_name,
+            'fee': 'fi',
+            'fo': 'fum',
+        }
+        self.projects_mock.update.assert_called_with(
+            identity_fakes.project_id,
+            **kwargs
+        )
+
 
 class TestProjectShow(TestProject):
 
